from __future__ import absolute_import, unicode_literals
from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^moderate/(?P<pk>\d+)/$',
        views.ModerateView.as_view(),
        name='moderate'),
    url(r'^moderate/(?P<pk>\d+)/(?P<post_id>.+)/allow/$',
        views.ModerateAllowView.as_view(),
        name='allow'),
    url(r'^moderate/(?P<pk>\d+)/(?P<post_id>.+)/remove/$',
        views.ModerateRemoveView.as_view(),
        name='remove'),
]



from __future__ import absolute_import, unicode_literals

from django.conf import settings

DEFAULTS = {
    'CONFIG': {},
    'CACHE_DURATION': 900,
}


def get_socialfeed_setting(name):
    return getattr(settings, 'WAGTAIL_SOCIALFEED_{}'.format(name),
                   DEFAULTS[name])



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-31 12:49
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0020_add_index_on_page_first_published_at'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModeratedItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('moderated', models.DateTimeField(auto_now_add=True)),
                ('posted', models.DateTimeField()),
                ('external_id', models.CharField(max_length=255)),
                ('content', models.TextField()),
            ],
            options={
                'ordering': ['-posted'],
            },
        ),
        migrations.CreateModel(
            name='SocialFeedConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('source', models.CharField(choices=[('twitter', 'Twitter'), ('instagram', 'Instagram')], max_length=100, verbose_name='Feed source')),
                ('username', models.CharField(max_length=255, verbose_name='User to track')),
                ('moderated', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='SocialFeedPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('feedconfig', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='wagtailsocialfeed.SocialFeedConfiguration')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
        migrations.AddField(
            model_name='moderateditem',
            name='config',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='moderated_items', to='wagtailsocialfeed.SocialFeedConfiguration'),
        ),
    ]



from dateutil import parser as dateparser
from django.core.exceptions import ImproperlyConfigured
from twython import Twython

from wagtailsocialfeed.utils.conf import get_socialfeed_setting

from . import AbstractFeed


def process_images(media_dict):
    images = {}
    if not media_dict:
        return images
    base_url = media_dict[0]['media_url_https']

    # TODO: see if we can provide the width and height attributes as well
    images['small'] = dict(url=base_url + ":small")
    images['thumb'] = dict(url=base_url + ":thumb")
    images['medium'] = dict(url=base_url + ":medium")
    images['large'] = dict(url=base_url + ":large")
    return images


def prepare_item(item):
    image = None
    extended = item.get('extended_entities', None)
    if extended:
        image = process_images(extended.get('media', None))

    item['image'] = image

    # Use the dateutil parser because on some platforms
    # python's own strptime doesn't support the %z directive.
    # Format: '%a %b %d %H:%M:%S %z %Y'
    item['date'] = dateparser.parse(item.get('created_at'))

    return item


class TwitterFeed(AbstractFeed):
    def __init__(self, *args, **kwargs):
        super(TwitterFeed, self).__init__(*args, **kwargs)

        self.settings = get_socialfeed_setting('CONFIG').get('twitter', None)
        if not self.settings:
            raise ImproperlyConfigured(
                "No twitter configuration defined in the settings. "
                "Make sure you define WAGTAIL_SOCIALFEED_CONFIG in your "
                "settings with at least a 'twitter' entry.")

    def fetch_online(self, config, limit=None):
        username = config.username
        twitter = Twython(self.settings['CONSUMER_KEY'],
                          self.settings['CONSUMER_SECRET'],
                          self.settings['ACCESS_TOKEN_KEY'],
                          self.settings['ACCESS_TOKEN_SECRET'])

        tweets = twitter.get_user_timeline(
            screen_name=username,
            trim_user=True,
            contributor_details=False,
            include_rts=False)
        return list(map(prepare_item, tweets))



from __future__ import unicode_literals

from django.conf.urls import include, url
from django.core.urlresolvers import reverse
from django.utils import six
from django.utils.translation import ugettext_lazy as _

from wagtail.contrib.modeladmin.menus import SubMenu
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.wagtailadmin.menu import MenuItem, SubmenuMenuItem
from wagtail.wagtailcore import hooks

from . import urls
from .models import SocialFeedConfiguration


class SocialFeedConfigurationAdmin(ModelAdmin):
    model = SocialFeedConfiguration
    menu_label = 'Social feeds'  # TODO ditch this to use verbose_name_plural from model # NOQA
    menu_icon = 'icon icon-fa-rss'
    menu_order = 400
    add_to_settings_menu = True
    list_display = ('source', 'username')
    list_filter = ('source', )

modeladmin_register(SocialFeedConfigurationAdmin)


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^socialfeed/', include(urls, app_name='wagtailsocialfeed',
            namespace='wagtailsocialfeed')),
    ]


@hooks.register('register_admin_menu_item')
def register_socialfeed_menu():
    config_menu_items = []
    config_qs = SocialFeedConfiguration.objects.filter(moderated=True)
    for config in config_qs:
        url = reverse('wagtailsocialfeed:moderate', kwargs={'pk': config.pk})
        config_menu_items.append(
            MenuItem(six.text_type(config), url,
                     classnames='icon icon-folder-inverse')
        )

    socialfeed_menu = SubMenu(config_menu_items)
    return SubmenuMenuItem(
        _('Social feed'), socialfeed_menu, classnames='icon icon-fa-rss',
        order=800)



from .twitter import TwitterFeed
from .instagram import InstagramFeed


class FeedFactory(object):
    @classmethod
    def create(cls, source):
        if source == 'twitter':
            return TwitterFeed()
        if source == 'instagram':
            return InstagramFeed()
        raise NotImplementedError(
            "Feed class for type '{}' not available".format(source))



from __future__ import absolute_import, unicode_literals

import json

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailadmin.edit_handlers import FieldPanel
from wagtail.wagtailcore.models import Page

from .utils.feed.factory import FeedFactory
from .managers import ModeratedItemManager


@python_2_unicode_compatible
class SocialFeedConfiguration(models.Model):
    FEED_CHOICES = (
        ('twitter', _('Twitter')),
        ('instagram', _('Instagram')),
    )

    source = models.CharField(_('Feed source'),
                              max_length=100,
                              choices=FEED_CHOICES,
                              blank=False)
    username = models.CharField(_('User to track'),
                                max_length=255,
                                blank=False)
    moderated = models.BooleanField(default=False)

    def __str__(self):
        name = self.username
        if self.source == 'twitter':
            name = "@{}".format(self.username)
        return "{} ({})".format(self.source, name)


class ModeratedItem(models.Model):
    config = models.ForeignKey(SocialFeedConfiguration,
                               related_name='moderated_items',
                               on_delete=models.CASCADE)
    moderated = models.DateTimeField(auto_now_add=True)
    posted = models.DateTimeField(blank=False, null=False)

    external_id = models.CharField(max_length=255,
                                   blank=False)
    content = models.TextField(blank=False)

    objects = ModeratedItemManager()

    class Meta:
        ordering = ['-posted', ]

    def get_content(self):
        return json.loads(self.content)


class SocialFeedPage(Page):
    feedconfig = models.ForeignKey(SocialFeedConfiguration,
                                   blank=False,
                                   null=False,
                                   on_delete=models.PROTECT)

    content_panels = Page.content_panels + [
        FieldPanel('feedconfig'),
    ]

    def get_context(self, request, *args, **kwargs):
        context = super(SocialFeedPage,
                        self).get_context(request, *args, **kwargs)

        if self.feedconfig.moderated:
            feed = self.feedconfig.moderated_items.all()
        else:
            stream = FeedFactory.create(self.feedconfig.source)
            feed = stream.get_feed(config=self.feedconfig, limit=20)

        context['feed'] = feed
        return context



import logging

from django.core.cache import cache

from wagtailsocialfeed.utils.conf import get_socialfeed_setting

logger = logging.getLogger('wagtailsocialfeed')


class FeedError(Exception):
    pass


class AbstractFeed(object):
    def get_feed(self, config, *args, **kwargs):
        cls_name = self.__class__.__name__
        cache_key = 'socialfeed:{}:data:{}'.format(cls_name, config.id)

        data = cache.get(cache_key, [])
        if data:
            logger.debug("Getting data from cache ({})".format(cache_key))
        else:
            logger.debug("Fetching data online")
            data = self.fetch_online(config=config, *args, **kwargs)
            logger.debug("Storing data in cache ({})".format(cache_key))
            cache.set(cache_key, data,
                      get_socialfeed_setting('CACHE_DURATION'))
        return data

    def fetch_online(self, *args, **kwargs):
        raise NotImplementedError(
            "fetch_online() is not implemented yet by {}".format(
                self.__class__.__name__))



import datetime

import requests
from django.utils import timezone

from . import AbstractFeed, FeedError


def prepare_item(item):
    date = None
    image = {}

    if 'created_time' in item:
        timestamp = None
        try:
            timestamp = float(item['created_time'])
        except ValueError:
            pass

        if timestamp:
            date = timezone.make_aware(
                datetime.datetime.fromtimestamp(timestamp),
                timezone=timezone.utc)

    if 'images' in item:
        image = {
            'thumb': item['images']['thumbnail'],
            'small': item['images']['low_resolution'],
            'medium': item['images']['standard_resolution'],
            'largel': None,
        }

    return {
        'id': item['id'],
        'text': item['caption']['text'],
        'image': image,
        'date': date,
        # 'instagram': item
        'type': 'instagram'
    }


class InstagramFeed(AbstractFeed):
    def fetch_online(self, config, limit=None):
        username = config.username

        url = "https://www.instagram.com/{}/media/".format(username)
        resp = requests.get(url)
        if resp.status_code == 200:
            try:
                items = resp.json()['items']
            except ValueError as e:
                raise FeedError(e)
            except KeyError as e:
                raise FeedError("No items could be found in the response")

            return list(map(prepare_item, items))

        raise FeedError(resp.reason)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as changelog_file:
    changelog = changelog_file.read()

install_requires = [
    'wagtail>=1.5',
    'twython>=3.0,<4.0',
    'wagtailfontawesome>=1.0',
    'requests>=2.0',
    'python-dateutil>=2.5'
]

test_require = [
    'responses>=0.5',
    'factory-boy>=2.7,<2.8',
    'beautifulsoup4>=4',
    'coverage>=3.7.0',
    'flake8>=2.2.0',
    'isort>=4.2.0',
    'tox>=2.3.1',
    'cryptography==1.4',
    'PyYAML==3.11',
    'bumpversion==0.5.3',
    'wheel==0.29.0',
]

docs_require = [
    'sphinx',
    'sphinx_rtd_theme',
]

setup(
    name='wagtailsocialfeed',
    version='0.1.dev1',
    description="A Wagtail module that provides pages and content blocks to show social media feeds", # NOQA
    long_description=readme + '\n\n' + changelog,
    author="Tim Leguijt",
    author_email='info@leguijtict.nl',
    url='https://github.com/LUKKIEN/wagtailsocialfeed',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=install_requires,
    license="",
    zip_safe=False,
    keywords='wagtailsocialfeed',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    extras_require={
        'testing': test_require,
        'docs': docs_require,
    },
)



from __future__ import unicode_literals

import json

from django.http import JsonResponse
from django.utils import six
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View
from django.views.generic.detail import DetailView

from .models import ModeratedItem, SocialFeedConfiguration
from .utils.feed.factory import FeedFactory


def date_handler(obj):
    if hasattr(obj, 'isoformat'):
        return obj.isoformat()
    else:
        raise TypeError


class ModerateView(DetailView):
    """
    ModerateView.

    Collect the latest feeds and find out which ones are already
    moderated/allowed in the feed (already have an `ModeratedItem`
    associated with them
    """

    page_title = _('Moderating')
    template_name = 'wagtailsocialfeed/admin/moderate.html'
    queryset = SocialFeedConfiguration.objects.filter(moderated=True)

    def get_context_data(self, **kwargs):
        context = super(ModerateView, self).get_context_data(**kwargs)

        feed = FeedFactory.create(self.object.source)
        items = feed.get_feed(config=self.object, limit=20)

        if self.object.moderated:
            allowed_ids = self.object.moderated_items.values_list(
                'external_id', flat=True)
            for item in items:
                # Provide JSON dump of the original item.
                # This is passed on along with the 'allow' POST action so that
                # we have the original post available without having to look
                # into the cache (might not be there) or request it from the
                # feed source again.
                item_json = json.dumps(item, default=date_handler)
                item['json'] = item_json

                # Flag to see if this item is already allowed in
                # the feed or not
                item['allowed'] = item['id'] in allowed_ids

        context['feed'] = items

        return context

error_messages = {
    'no_original':
        _('The original social feed post was not found in the POST data'),
    'not_found': _('The moderated item with the given id could not be found')
}


class ModerateAllowView(View):
    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super(ModerateAllowView, self).dispatch(*args, **kwargs)

    def post(self, request, pk, post_id):
        config = SocialFeedConfiguration.objects.get(pk=pk)

        if 'original' not in request.POST:
            err = {'message': six.text_type(error_messages['no_original'])}
            return JsonResponse(err, status=400)

        original = request.POST['original']
        item, created = config.moderated_items.get_or_create_for(original)

        return JsonResponse({
            'message': 'The post is now allowed on the feed',
            'post_id': post_id,
            'allowed': True
        })


class ModerateRemoveView(View):
    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        return super(ModerateRemoveView, self).dispatch(*args, **kwargs)

    def post(self, request, pk, post_id):
        config = SocialFeedConfiguration.objects.get(pk=pk)

        try:
            item = config.moderated_items.get(external_id=post_id)
        except ModeratedItem.DoesNotExist:
            err = {'message': six.text_type(error_messages['not_found'])}
            return JsonResponse(err, status=404)

        item.delete()

        return JsonResponse({
            'message': 'The post is removed from the feed',
            'post_id': post_id,
            'allowed': False
        })



import json

import dateutil.parser
from django.db import models


class ModeratedItemManager(models.Manager):
    def get_or_create_for(self, original_post):
        """
        Get an existing `ModeratedItem` based on the original_post
        or create a new one if it cannot be found.

        :param original_post:
            The original post as a JSON string or encoded JSON object
        """
        original_obj = json.loads(original_post)

        created = dateutil.parser.parse(original_obj['date'])
        external_id = original_obj['id']
        return self.get_or_create(
            external_id=external_id,
            defaults=dict(
                posted=created,
                external_id=external_id,
                content=original_post))
